@@ -1,21 +1,23 @@
-import os
 import asyncio
 import fnmatch
 import json
 import logging
+import os
+import typing
 from functools import partial
-from os import path, listdir
 from urllib.parse import urlparse
 
 from aiohttp import web
 
+from . import page
+from .adaptor import ws as ws_adaptor
+from .page import make_applications, render_page
 from .remote_access import start_remote_access_service
 from .tornado import open_webbrowser_on_server_started
-from .page import make_applications, render_page
-from .utils import cdn_validation, deserialize_binary_event, print_listen_address
-from ..session import CoroutineBasedSession, ThreadBasedSession, register_session_implement_for_target, Session
+from .utils import cdn_validation, print_listen_address
+from ..session import register_session_implement_for_target, Session
 from ..session.base import get_session_info_from_headers
-from ..utils import get_free_port, STATIC_PATH, iscoroutinefunction, isgeneratorfunction
+from ..utils import get_free_port, STATIC_PATH, parse_file_size
 
 logger = logging.getLogger(__name__)
 
@@ -40,15 +42,47 @@ def _is_same_site(origin, host):
     return origin == host
 
 
-def _webio_handler(applications, cdn, websocket_settings, check_origin_func=_is_same_site):
+class WebSocketConnection(ws_adaptor.WebSocketConnection):
+
+    def __init__(self, ws: web.WebSocketResponse, http: web.Request, ioloop):
+        self.ws = ws
+        self.http = http
+        self.ioloop = ioloop
+
+    def get_query_argument(self, name) -> typing.Optional[str]:
+        return self.http.query.getone(name, None)
+
+    def make_session_info(self) -> dict:
+        session_info = get_session_info_from_headers(self.http.headers)
+        session_info['user_ip'] = self.http.remote
+        session_info['request'] = self.http
+        session_info['backend'] = 'aiohttp'
+        session_info['protocol'] = 'websocket'
+        return session_info
+
+    def write_message(self, message: dict):
+        msg_str = json.dumps(message)
+        self.ioloop.create_task(self.ws.send_str(msg_str))
+
+    def closed(self) -> bool:
+        return self.ws.closed
+
+    def close(self):
+        self.ioloop.create_task(self.ws.close())
+
+
+def _webio_handler(applications, cdn, websocket_settings, reconnect_timeout=0, check_origin_func=_is_same_site):
     """
     :param dict applications: dict of `name -> task function`
     :param bool/str cdn: Whether to load front-end static resources from CDN
     :param callable check_origin_func: check_origin_func(origin, host) -> bool
     :return: aiohttp Request Handler
     """
+    ws_adaptor.set_expire_second(reconnect_timeout)
+
     async def wshandle(request: web.Request):
         ioloop = asyncio.get_event_loop()
+        asyncio.get_event_loop().create_task(ws_adaptor.session_clean_task())
 
         origin = request.headers.get('origin')
         if origin and not check_origin_func(origin=origin, host=request.host):
@@ -61,67 +95,38 @@ def _webio_handler(applications, cdn, websocket_settings, check_origin_func=_is_
 
             app_name = request.query.getone('app', 'index')
             app = applications.get(app_name) or applications['index']
-            html = render_page(app, protocol='ws', cdn=cdn)
+            no_cdn = cdn is True and request.query.getone('_pywebio_cdn', '') == 'false'
+            html = render_page(app, protocol='ws', cdn=False if no_cdn else cdn)
             return web.Response(body=html, content_type='text/html')
 
         ws = web.WebSocketResponse(**websocket_settings)
         await ws.prepare(request)
 
-        close_from_session_tag = False  # 是否由session主动关闭连接
-
-        def send_msg_to_client(session: Session):
-            for msg in session.get_task_commands():
-                msg_str = json.dumps(msg)
-                ioloop.create_task(ws.send_str(msg_str))
-
-        def close_from_session():
-            nonlocal close_from_session_tag
-            close_from_session_tag = True
-            ioloop.create_task(ws.close())
-            logger.debug("WebSocket closed from session")
-
-        session_info = get_session_info_from_headers(request.headers)
-        session_info['user_ip'] = request.remote
-        session_info['request'] = request
-        session_info['backend'] = 'aiohttp'
-        session_info['protocol'] = 'websocket'
-
         app_name = request.query.getone('app', 'index')
         application = applications.get(app_name) or applications['index']
 
-        if iscoroutinefunction(application) or isgeneratorfunction(application):
-            session = CoroutineBasedSession(application, session_info=session_info,
-                                            on_task_command=send_msg_to_client,
-                                            on_session_close=close_from_session)
-        else:
-            session = ThreadBasedSession(application, session_info=session_info,
-                                         on_task_command=send_msg_to_client,
-                                         on_session_close=close_from_session, loop=ioloop)
+        conn = WebSocketConnection(ws, request, ioloop)
+        handler = ws_adaptor.WebSocketHandler(
+            connection=conn, application=application, reconnectable=bool(reconnect_timeout), ioloop=ioloop
+        )
 
         # see: https://github.com/aio-libs/aiohttp/issues/1768
         try:
             async for msg in ws:
-                if msg.type == web.WSMsgType.text:
-                    data = msg.json()
-                elif msg.type == web.WSMsgType.binary:
-                    data = deserialize_binary_event(msg.data)
+                if msg.type in (web.WSMsgType.text, web.WSMsgType.binary):
+                    handler.send_client_data(msg.data)
                 elif msg.type == web.WSMsgType.close:
                     raise asyncio.CancelledError()
-
-                if data is not None:
-                    session.send_client_event(data)
         finally:
-            if not close_from_session_tag:
-                # close session because client disconnected to server
-                session.close(nonblock=True)
-                logger.debug("WebSocket closed from client")
+            handler.notify_connection_lost()
 
         return ws
 
     return wshandle
 
 
-def webio_handler(applications, cdn=True, allowed_origins=None, check_origin=None, websocket_settings=None):
+def webio_handler(applications, cdn=True, reconnect_timeout=0, allowed_origins=None, check_origin=None,
+                  max_payload_size='200M', websocket_settings=None):
     """Get the `Request Handler <https://docs.aiohttp.org/en/stable/web_quickstart.html#aiohttp-web-handler>`_ coroutine for running PyWebIO applications in aiohttp.
     The handler communicates with the browser by WebSocket protocol.
 
@@ -135,6 +140,9 @@ def webio_handler(applications, cdn=True, allowed_origins=None, check_origin=Non
 
     websocket_settings = websocket_settings or {}
 
+    page.MAX_PAYLOAD_SIZE = max_payload_size = parse_file_size(max_payload_size)
+    websocket_settings.setdefault('max_msg_size', max_payload_size)
+
     cdn = cdn_validation(cdn, 'error')
 
     if check_origin is None:
@@ -144,6 +152,7 @@ def webio_handler(applications, cdn=True, allowed_origins=None, check_origin=Non
 
     return _webio_handler(applications=applications, cdn=cdn,
                           check_origin_func=check_origin_func,
+                          reconnect_timeout=reconnect_timeout,
                           websocket_settings=websocket_settings)
 
 
@@ -154,25 +163,23 @@ def static_routes(prefix='/'):
     :param str prefix: The URL path of static file hosting, the default is the root path ``/``
     :return: aiohttp routes list
     """
-
-    async def index(request):
-        return web.FileResponse(path.join(STATIC_PATH, 'index.html'))
-
-    files = [path.join(STATIC_PATH, d) for d in listdir(STATIC_PATH)]
-    dirs = filter(path.isdir, files)
-    routes = [web.static(prefix + path.basename(d), d) for d in dirs]
-    routes.append(web.get(prefix, index))
+    files = [os.path.join(STATIC_PATH, d) for d in os.listdir(STATIC_PATH)]
+    dirs = filter(os.path.isdir, files)
+    routes = [web.static(prefix + os.path.basename(d), d) for d in dirs]
     return routes
 
 
 def start_server(applications, port=0, host='', debug=False,
                  cdn=True, static_dir=None, remote_access=False,
+                 reconnect_timeout=0,
                  allowed_origins=None, check_origin=None,
                  auto_open_webbrowser=False,
+                 max_payload_size='200M',
                  websocket_settings=None,
                  **aiohttp_settings):
     """Start a aiohttp server to provide the PyWebIO application as a web service.
 
+
     :param dict websocket_settings: The  parameters passed to the constructor of ``aiohttp.web.WebSocketResponse``.
        For details, please refer: https://docs.aiohttp.org/en/stable/web_reference.html#websocketresponse
     :param aiohttp_settings: Additional keyword arguments passed to the constructor of ``aiohttp.web.Application``.
@@ -190,8 +197,9 @@ def start_server(applications, port=0, host='', debug=False,
 
     cdn = cdn_validation(cdn, 'warn')
 
-    handler = webio_handler(applications, cdn=cdn, allowed_origins=allowed_origins,
-                            check_origin=check_origin, websocket_settings=websocket_settings)
+    handler = webio_handler(applications, cdn=cdn, allowed_origins=allowed_origins, reconnect_timeout=reconnect_timeout,
+                            check_origin=check_origin, max_payload_size=max_payload_size,
+                            websocket_settings=websocket_settings)
 
     app = web.Application(**aiohttp_settings)
     app.router.add_routes([web.get('/', handler)])
@@ -200,7 +208,7 @@ def start_server(applications, port=0, host='', debug=False,
     app.router.add_routes(static_routes())
 
     if auto_open_webbrowser:
-        asyncio.get_event_loop().create_task(open_webbrowser_on_server_started('localhost', port))
+        asyncio.get_event_loop().create_task(open_webbrowser_on_server_started('127.0.0.1', port))
 
     debug = Session.debug = os.environ.get('PYWEBIO_DEBUG', debug)
     if debug:
