@@ -5,10 +5,12 @@ import threading
 
 from django.http import HttpResponse, HttpRequest
 
-from . import utils
-from .httpbased import HttpContext, HttpHandler, run_event_loop
+from . import page
+from ..session import Session
+from .adaptor.http import HttpContext, HttpHandler, run_event_loop
 from .remote_access import start_remote_access_service
-from .utils import make_applications, cdn_validation
+from .page import make_applications
+from .utils import cdn_validation, print_listen_address
 from ..utils import STATIC_PATH, iscoroutinefunction, isgeneratorfunction, get_free_port, parse_file_size
 
 logger = logging.getLogger(__name__)
@@ -100,11 +102,11 @@ urlpatterns = []
 
 
 def wsgi_app(applications, cdn=True,
-                 static_dir=None,
-                 allowed_origins=None, check_origin=None,
-                 session_expire_seconds=None,
-                 session_cleanup_interval=None,
-                 debug=False, max_payload_size='200M', **django_options):
+             static_dir=None,
+             allowed_origins=None, check_origin=None,
+             session_expire_seconds=None,
+             session_cleanup_interval=None,
+             debug=False, max_payload_size='200M', **django_options):
     """Get the Django WSGI app for running PyWebIO applications.
 
     The arguments of ``wsgi_app()`` have the same meaning as for :func:`pywebio.platform.django.start_server`
@@ -118,9 +120,10 @@ def wsgi_app(applications, cdn=True,
     from django.views.static import serve
 
     cdn = cdn_validation(cdn, 'warn')
+    debug = Session.debug = os.environ.get('PYWEBIO_DEBUG', debug)
 
     max_payload_size = parse_file_size(max_payload_size)
-    utils.MAX_PAYLOAD_SIZE = max_payload_size
+    page.MAX_PAYLOAD_SIZE = max_payload_size
 
     django_options.update(dict(
         DEBUG=debug,
@@ -198,9 +201,10 @@ def start_server(applications, port=8080, host='', cdn=True,
                    session_cleanup_interval=session_cleanup_interval,
                    debug=debug, max_payload_size=max_payload_size, **django_options)
 
-    if remote_access or remote_access == {}:
-        if remote_access is True: remote_access = {}
-        start_remote_access_service(**remote_access, local_port=port)
+    print_listen_address(host, port)
+
+    if remote_access:
+        start_remote_access_service(local_port=port)
 
     use_tornado_wsgi = os.environ.get('PYWEBIO_DJANGO_WITH_TORNADO', True)
     if use_tornado_wsgi:
